#include "HighResolutionTimer.h"

#include <avr/io.h>
#include <avr/interrupt.h>

static uint32_t timerHigh = 0;
static uint32_t timerLow = 0;
static int isRunning = 0;

ISR(TCB1_INT_vect) {
	PORTD.OUTTGL = PIN2_bm;
	uint8_t flags = TCB1.INTFLAGS;
	if (flags & TCB_OVF_bm) {
		timerHigh++;
	}
	if ((flags & TCB_CAPT_bm) && isRunning) {
		timerLow = TCB1.CCMP;
		isRunning = 0;
	}
	TCB1.INTFLAGS = flags;
}

void HighResolutionTimer_Init() {
	// configure event system for attaching PA3 level to TCB1
	EVSYS.CHANNEL1 = EVSYS_CHANNEL1_PORTA_PIN3_gc;
	EVSYS.USERTCB1CAPT = 2;
	
	TCA0.SINGLE.CTRLA = TCA_SINGLE_CLKSEL_DIV16_gc | TCA_SINGLE_ENABLE_bm;
	
	TCB1.CTRLB = TCB_CNTMODE_CAPT_gc;
	TCB1.EVCTRL = TCB_CAPTEI_bm; // | TCB_EDGE_bm;
	TCB1.INTCTRL = TCB_OVF_bm | TCB_CAPT_bm;
	TCB1.CTRLA = TCB_CLKSEL_TCA0_gc | TCB_ENABLE_bm;
}

void HighResolutionTimer_Start() {
	isRunning = 1;
	
	TCB1.CNT = 0;
	
	timerHigh = 0;
	timerLow = 0;
}

int HighResolutionTimer_IsRunning() {
	return isRunning;
}

uint32_t HighResolutionTimer_GetTime() {
	return (timerHigh << 16) | timerLow;
}

uint32_t HighResolutionTimer_GetOverflowCount() {
	return timerHigh >> 16;
}