#include "UART.h"

#include <stddef.h>
#include <string.h>

void UART_Init() {
	UART_TX_PORT.DIRSET = UART_TX_PINMASK;
	
	UART_INST.CTRLB = USART_TXEN_bm;
	UART_INST.BAUD = 139;
	
	UART_SetFontColor(UART_COLOR_DEFAULT);
	UART_SetBackgroundColor(UART_COLOR_DEFAULT);
}

void UART_PrintString(char* message) {
	size_t len = strlen(message);
	for (int i = 0; i < len; i++) {
		while (!(UART_INST.STATUS & USART_DREIF_bm));
		UART_INST.STATUS |= USART_TXCIF_bm;
		UART_INST.TXDATAL = message[i];
	}
}

void UART_PrintNumber(int number) {
	unsigned char buff[12];
	itoa(number, buff, 10);
	UART_PrintString(buff);
}

void UART_PrintNumberHex(int number) {
	unsigned char buff[9];
	itoa(number, buff, 16);
	UART_PrintString(buff);
}

void UART_PrintNumberHex2(int number) {
	unsigned char buff[9];
	itoa(number, buff, 16);
	if (strlen(buff) == 1) {
		UART_PrintString("0");
	}
	UART_PrintString(buff);
}

void UART_PrintNumberHex4(int number) {
	unsigned char buff[9];
	itoa(number, buff, 16);
	int len = strlen(buff);
	
	if (len == 1) {
		UART_PrintString("000");
	} else if (len == 2) {
		UART_PrintString("00");
	} else if (len == 3) {
		UART_PrintString("0");
	}
	
	UART_PrintString(buff);
}

void UART_PrintNumberBin(int number) {
	unsigned char buff[33];
	itoa(number, buff, 2);
	UART_PrintString(buff);
}

void UART_SetFontColor(uint8_t color) {
	UART_PrintString("\033[1;3");
	UART_PrintNumber(color);
	UART_PrintString("m");
}

void UART_SetBackgroundColor(uint8_t color) {
	UART_PrintString("\033[4");
	UART_PrintNumber(color);
	UART_PrintString("m");
}
